@@ -66,15 +66,15 @@
 #' is fixed, and the position of its surrounding positions will not be
 #' searched.
 #' @param speed numeric. The walking speed of the QTL search (in cM).
-#' @param conv numeric. The convergent criterion of EM algorithm.
-#' The E and M steps will be iterated until a convergent criterion
+#' @param conv numeric. The convergence criterion of EM algorithm.
+#' The E and M steps will be iterated until a convergence criterion
 #' is satisfied.
 #' @param console logical. To decide whether the process of algorithm will
 #' be shown in the R console or not.
 #'
 #' @return
-#' \item{effect}{The estimated effects and LRT statistics of all searched
-#' positions.}
+#' \item{effect}{The estimated effects, log likelihood value, and LRT
+#' statistics of all searched positions.}
 #' \item{QTL.best}{The positions of the best QTL combination.}
 #' \item{effect.best}{The estimated effects and LRT statistics of the best
 #' QTL combination.}
@@ -261,7 +261,7 @@ MIM.points2 <- function(QTL, marker, geno, y, yu = NULL, sele.g = "n", tL = NULL
   }
 
   name0 <- cbind(paste("QTL", 1:nq, ".ch", sep = ""), paste("QTL", 1:nq, ".cM", sep = ""))
-  if(console){cat("#", t(name0), "LRT", "\n", sep = "\t")}
+  if(console){cat("#", t(name0), "LRT", "log.likelihood", "\n", sep = "\t")}
 
   if(method == "EM"){
     meth <- function(QTL, marker, geno, D.matrix, y, yu, tL, tR, type, ng, sele.g, conv){
@@ -271,8 +271,10 @@ MIM.points2 <- function(QTL, marker, geno, y, yu = NULL, sele.g = "n", tL = NULL
       mu0 <- as.numeric(EM$beta)
       sigma <- sqrt(as.numeric(EM$variance))
       LRT <- EM$LRT
+      like <- EM$log.likelihood
+      R2 <- EM$R2
       model <- EM$model
-      result <- list(eff, mu0, sigma, LRT, model)
+      result <- list(eff, mu0, sigma, LRT, like, R2, model)
       return(result)
     }
     if(sele.g == "p" | sele.g == "f"){
@@ -441,6 +443,7 @@ MIM.points2 <- function(QTL, marker, geno, y, yu = NULL, sele.g = "n", tL = NULL
       mu0 <- as.numeric(fit$coefficients[1])
       ms <- stats::anova(fit)$`Mean Sq`
       sigma <- ms[2]^0.5
+      R2 <- summary(fit)$r.squared
 
       L0 <- c()
       L1 <- c()
@@ -454,27 +457,31 @@ MIM.points2 <- function(QTL, marker, geno, y, yu = NULL, sele.g = "n", tL = NULL
         L0[k] <- sum(L00)
         L1[k] <- sum(L01)
       }
-      LRT <- -2*sum(log(L0[!is.na(L0) & !is.na(L1)]/L1[!is.na(L0) & !is.na(L1)]))
+      like0 <- sum(log(L0))
+      like1 <- sum(log(L1))
+      LRT <- 2*(like1-like0)
 
-      result <- list(eff, mu0, sigma, LRT, model = "regression interval mapping model")
+      result <- list(eff, mu0, sigma, LRT, like1, R2, model = "regression interval mapping model")
       return(result)
     }
   }
 
-  effect <- matrix(NA, nrow(sm), 2*nq+ncol(D.matrix)+1)
+  effect <- matrix(NA, nrow(sm), 2*nq+ncol(D.matrix)+3)
   for(i in 1:nrow(sm)){
     QTL0 <- cbind(ch0, sm[i,])
     fit0 <- meth(QTL0, marker, geno, D.matrix, y, yu, tL, tR, type, ng, sele.g, conv)
-    effect0 <- c(t(QTL0), fit0[[1]], fit0[[4]])
-    model <- fit0[[5]]
+    effect0 <- c(t(QTL0), fit0[[1]], fit0[[4]], fit0[[5]], fit0[[6]])
+    model <- fit0[[7]]
 
-    if(console){cat(paste(i, "/", nrow(sm), sep = ""), t(QTL0), effect0[length(effect0)], "\n", sep = "\t")}
+    LRT <- round(effect0[length(effect0)-2], 3)
+    like <- round(effect0[length(effect0)-1], 3)
+    if(console){cat(paste(i, "/", nrow(sm), sep = ""), t(QTL0), LRT, like, "\n", sep = "\t")}
     effect[i,] <- effect0
   }
-  colnames(effect) <-  c(t(name0), colnames(D.matrix), "LRT")
+  colnames(effect) <-  c(t(name0), colnames(D.matrix), "LRT", "log.likelihood", "R2")
   row.names(effect) <- 1:nrow(effect)
 
-  best <- effect[effect[,ncol(effect)] == max(effect[,ncol(effect)]),]
+  best <- effect[effect[,ncol(effect)-1] == max(effect[,ncol(effect)-1]),]
   QTL.best <- matrix(best[1:(2*nq)], nq, 2, byrow = TRUE)
   colnames(QTL.best) <- c("chromosome", "position(cM)")
 
